/*  Sensor Platform v1.2
 *  
 *  This sketch was created for a mobile sensing platform centered around the Arduino Uno 
 *  using a DHT temperature/humidity sensor, an Adafruit_GPS compatible GPS chip, and an
 *  SD card. The program polls the DHT sensor and GPS every 3 seconds - the wait is 
 *  necessitated by the DHT sensor, which needs time between polling - and writes the 
 *  results to an SD card. 
 */

//Libraries needed
#include <avr/pgmspace.h>
#include <SD.h> //Load SD card library
#include<SPI.h> //Load SPI Library
#include "DHT.h"
#include <Adafruit_GPS.h> 
#include <SoftwareSerial.h>

// BEGIN INITIALIZATION

SoftwareSerial mySerial(8, 7); //Initialize SoftwareSerial, and tell it you will be connecting through pins 2 and 3
Adafruit_GPS GPS(&mySerial); //Create GPS object

// Set GPSECHO to 'false' to turn off echoing the GPS data to the Serial console
// Set to 'true' if you want to debug and listen to the raw GPS sentences
#define GPSECHO  true
/* set to true to only log to SD when GPS has a fix, for debugging, keep it false */
#define LOG_FIXONLY false  

String NMEA1;  //We will use this variable to hold our first NMEA sentence
String NMEA2;  //We will use this variable to hold our second NMEA sentence

#define chipSelect 10
#define ledPin 13

char c;       //Used to read the characters spewing from the GPS module

File mySensorData; //Data object you will write your sensor data to for temp

#define DHTPIN 9     // what pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302)

DHT dht(DHTPIN, DHTTYPE);

// Variables for data that goes on SD
float humidity, temp_c, temp_f;
float prev_temp = 0;
float diff_temp = 0;
// END INITIALIZATION

void setup()  {

  Serial.begin(115200);
  pinMode(ledPin, OUTPUT);

  //GPS initialization
  GPS.begin(9600);       //Turn GPS on at baud rate of 9600
  GPS.sendCommand("$PGCMD,33,0*6D"); // Turn Off GPS Antenna Update
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA); //Tell GPS we want only $GPRMC and $GPGGA NMEA sentences
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);   // 1 Hz update rate

  //SD initialization
  delay(2000);  //Pause; this is required by the temperature sensor
  pinMode(10, OUTPUT); //Must declare 10 an output and reserve it to keep SD card happy
  SD.begin(chipSelect); //Initialize the SD card reader

  //Temperature & humidity sensor initialization
  dht.begin();

  mySensorData = SD.open("Ldata.txt", FILE_WRITE);

  //Print file header
  mySensorData.println("UnitCode, Date, Time, Latitude, N-S, Longitude, E-W, Elevation, Humidity, Temperature(C), Temperature(F), QC");
  mySensorData.close(); //close file
}

void loop() {   // run over and over again

  //Temperature & humidity sensor requires a minimum of 3 seconds
  //between data call; this sketch uses a 5 second delay - the
  //2 second delay above and the 3 second delay here - as that
  //seems to result in the most reliable reporting from the sensor
  delay(3000);

  //Open data file
  mySensorData = SD.open("Ldata.txt", FILE_WRITE);
  //Functions below take data from sensors & write to SD
  readGPS();  //Read GPS, write data
  tempread();  // Read DHT sensor, write data
  
  //Close file
  mySensorData.close(); 
}


void readGPS(){  //This function will read and remember two NMEA sentences from GPS

  //Collect and check GPS data
  clearGPS();    //Serial port probably has old or corrupt data, so begin by clearing it all out
  while(!GPS.newNMEAreceived()) { //Keep reading characters in this loop until a good NMEA sentence is received
    c=GPS.read(); //read a character from the GPS
  }
  GPS.parse(GPS.lastNMEA());  //Once you get a good NMEA, parse it
  NMEA1=GPS.lastNMEA();      //Once parsed, save NMEA sentence into NMEA1
  while(!GPS.newNMEAreceived()) {  //Go out and get the second NMEA sentence, should be different type than the first one read above.
    c=GPS.read();
  }
  GPS.parse(GPS.lastNMEA());
  NMEA2=GPS.lastNMEA();

    mySensorData.print("L,");
    //Print date
    mySensorData.print(GPS.year, DEC); 
    mySensorData.print('-');
    mySensorData.print(GPS.month, DEC); 
    mySensorData.print("-");
    mySensorData.print(GPS.day, DEC);
    mySensorData.print(",");

    //Print time
    mySensorData.print(GPS.hour, DEC); 
    mySensorData.print(':');
    mySensorData.print(GPS.minute, DEC); 
    mySensorData.print(':');
    mySensorData.print(GPS.seconds, DEC); 
    mySensorData.print(",");
   

  //Format and save data to file
  if(GPS.fix==1) { //Only save data if we have a fix

    //String lat = floatToString(GPS.latitude);
    //String longi = floatToString(GPS.longitude);

    String lat = String(GPS.latitude, 8);
    String longi = String(GPS.longitude);

    String degree = lat.substring(0, 2);
    String minutes = lat.substring(2, lat.length() -1);
    float fMinutes = 0.1;
    fMinutes = minutes.toFloat();
    float fDegree = 0.1;
    fDegree = degree.toFloat();
    fDegree = fDegree + (fMinutes / 60);

    String degreeL = longi.substring(0, 2);
    String minutesL = longi.substring(2, longi.length() -1);
    float fMinutesL = 0.1;
    fMinutesL = minutesL.toFloat();
    float fDegreeL = 0.1;
    fDegreeL = degreeL.toFloat();
    Serial.println(fDegreeL, 5);
    fDegreeL = fDegreeL + (fMinutesL / 60);

    //Print lat, long, and elevation
    //mySensorData.print(lat);
    //mySensorData.print(",");
    //mySensorData.print(GPS.longitude);
    //mySensorData.print(",");
    mySensorData.print(fDegree, 5); //Write measured latitude to file
    mySensorData.print(",");
    mySensorData.print(GPS.lat); //Which hemisphere N or S
    mySensorData.print(",");
    //mySensorData.print(longi);
    //Serial.println(longi);
    mySensorData.print(fDegreeL, 5); //Write measured longitude to file
    mySensorData.print(",");
    mySensorData.print(GPS.lon); //Which Hemisphere E or W
    mySensorData.print(",");
    mySensorData.print(GPS.altitude);
    mySensorData.print(",");

  }
  //Indicate if no fix is found --- this means no coordinates, elevation, or time
  if(GPS.fix != 1) {
    mySensorData.print(F("-999,"));
    mySensorData.print(F("-999,"));
    mySensorData.print(F("-999,"));  
    mySensorData.print(F("-999,"));
    mySensorData.print(F("-999,"));
  }
  digitalWrite(A3, 0);
}

void tempread() {
  
  // Reading temperature or humidity takes about 2000 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)

  // Read humidity
  float h = dht.readHumidity();
  // Read temperature as Celsius
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit
  float f = dht.readTemperature(true);
  prev_temp = temp_c;
  temp_c=t;
  temp_f=f;
  humidity= h;  

  diff_temp = prev_temp - temp_c;

if (temp_c < 0 || temp_c > 0 || temp_c == 0) {
  
  //Print data
  mySensorData.print(humidity);
  mySensorData.print(",");
  mySensorData.print(temp_c); //Print *C temp
  mySensorData.print(",");
  mySensorData.print(temp_f); //Print *F temp
  mySensorData.println(",");
}
else {
  mySensorData.println("-999,-999,-999,");
}
}

void clearGPS() {  //Since between GPS reads, we still have data streaming in, we need to clear the old data by reading a few sentences, and discarding these
  while(!GPS.newNMEAreceived()) {
    c=GPS.read();
  }
  GPS.parse(GPS.lastNMEA());
  while(!GPS.newNMEAreceived()) {
    c=GPS.read();
  }
  GPS.parse(GPS.lastNMEA());
}
